@@ -417,6 +417,7 @@ extern struct litest_test_device litest_lid_switch_device;
 extern struct litest_test_device litest_lid_switch_surface3_device;
 extern struct litest_test_device litest_appletouch_device;
 extern struct litest_test_device litest_gpio_keys_device;
+extern struct litest_test_device litest_wacom_mobilestudio_13hdt_pad_device;
 
 struct litest_test_device* devices[] = {
 	&litest_synaptics_clickpad_device,
@@ -483,6 +484,7 @@ struct litest_test_device* devices[] = {
 	&litest_lid_switch_surface3_device,
 	&litest_appletouch_device,
 	&litest_gpio_keys_device,
+	&litest_wacom_mobilestudio_13hdt_pad_device,
 	NULL,
 };
 
